import java.util.*;
import java.io.*;

public class MaxClique {

    static int[] degree;   // degree of vertices
    static int[][] A;      // 0/1 adjacency matrix
    static int n;

    static void readDIMACS(String fname) throws IOException {
	String s = "";
	Scanner sc = new Scanner(new File(fname));
	while (sc.hasNext() && !s.equals("p")) s = sc.next();
	sc.next();
	n      = sc.nextInt();   
	int m  = sc.nextInt();   
	degree = new int[n];
	A      = new int[n][n];
	while (sc.hasNext()){
	    s     = sc.next(); // skip "edge"
	    int i = sc.nextInt() - 1;
	    int j = sc.nextInt() - 1; 
	    degree[i]++; degree[j]++;
	    A[i][j] = A[j][i] = 1;
	}
	sc.close();
    }

    public static void main(String[] args)  throws IOException {

	readDIMACS(args[1]);
	MC mc = null;
	if (args[0].equals("MC0"))        mc = new MC(n,A,degree,0);
	else if (args[0].equals("MC-1"))   mc = new MC(n,A,degree,-1);
	else if (args[0].equals("MC-2"))   mc = new MC(n,A,degree,-2);	
	else if (args[0].equals("MC-3"))   mc = new MC(n,A,degree,-3);
	else if (args[0].equals("MC1"))   mc = new MC(n,A,degree,1);
	else if (args[0].equals("MC3"))   mc = new MC(n,A,degree,2);
	else if (args[0].equals("MC2"))   mc = new MC(n,A,degree,3);
	else if (args[0].equals("MCQ1"))  mc = new MCQ(n,A,degree,1);
	else if (args[0].equals("MCQ2"))  mc = new MCQ(n,A,degree,2);
	else if (args[0].equals("MCQ3"))  mc = new MCQ(n,A,degree,3);
	else if (args[0].equals("MCSa1")) mc = new MCSa(n,A,degree,1);
	else if (args[0].equals("MCSa2")) mc = new MCSa(n,A,degree,2);
	else if (args[0].equals("MCSa3")) mc = new MCSa(n,A,degree,3);
	else if (args[0].equals("MCSb1")) mc = new MCSb(n,A,degree,1);
	else if (args[0].equals("MCSb2")) mc = new MCSb(n,A,degree,2);
	else if (args[0].equals("MCSb3")) mc = new MCSb(n,A,degree,3);
	else if (args[0].equals("BBMC1")) mc = new BBMC(n,A,degree,1);
	else if (args[0].equals("BBMC2")) mc = new BBMC(n,A,degree,2);
	else if (args[0].equals("BBMC3")) mc = new BBMC(n,A,degree,3);
	else if (args[0].equals("PPMC1")) mc = new PPMC(n,A,degree,1);
	else if (args[0].equals("PPMC2")) mc = new PPMC(n,A,degree,2);
	else if (args[0].equals("MCSaR1")) mc = new MCSaR(n,A,degree,1);
	else if (args[0].equals("MCSaR2")) mc = new MCSaR(n,A,degree,2);
	else if (args[0].equals("MCSaR3")) mc = new MCSaR(n,A,degree,3);
	else if (args[0].equals("MCSbR1")) mc = new MCSbR(n,A,degree,1);
	else if (args[0].equals("MCSbR2")) mc = new MCSbR(n,A,degree,2);
	else if (args[0].equals("MCSbR3")) mc = new MCSbR(n,A,degree,3);
	else if (args[0].equals("MCCol0"))    mc = new MCCol(n,A,degree,0);
	else if (args[0].equals("MCCol-1"))   mc = new MCCol(n,A,degree,-1);
	else if (args[0].equals("MCCol-2"))   mc = new MCCol(n,A,degree,-2);	
	else if (args[0].equals("MCCol-3"))   mc = new MCCol(n,A,degree,-3);
	else if (args[0].equals("MCCol1"))    mc = new MCCol(n,A,degree,1);
	else if (args[0].equals("MCCol2"))    mc = new MCCol(n,A,degree,2);
	else if (args[0].equals("MCCol3"))    mc = new MCCol(n,A,degree,3);
	else if (args[0].equals("MCColPivotF0"))    mc = new MCColPivot(n,A,degree,-0,0);
	else if (args[0].equals("MCColPivotF-1"))   mc = new MCColPivot(n,A,degree,-1,0);
	else if (args[0].equals("MCColPivotF-2"))   mc = new MCColPivot(n,A,degree,-2,0);	
	else if (args[0].equals("MCColPivotF-3"))   mc = new MCColPivot(n,A,degree,-3,0);
	else if (args[0].equals("MCColPivotF1"))    mc = new MCColPivot(n,A,degree,1,0);
	else if (args[0].equals("MCColPivotF2"))    mc = new MCColPivot(n,A,degree,2,0);
	else if (args[0].equals("MCColPivotF3"))    mc = new MCColPivot(n,A,degree,3,0);
	else if (args[0].equals("MCColPivotMin0"))    mc = new MCColPivot(n,A,degree,-0,2);
	else if (args[0].equals("MCColPivotMin-1"))   mc = new MCColPivot(n,A,degree,-1,2);
	else if (args[0].equals("MCColPivotMin-2"))   mc = new MCColPivot(n,A,degree,-2,2);	
	else if (args[0].equals("MCColPivotMin-3"))   mc = new MCColPivot(n,A,degree,-3,2);
	else if (args[0].equals("MCColPivotMin1"))    mc = new MCColPivot(n,A,degree,1,2);
	else if (args[0].equals("MCColPivotMin2"))    mc = new MCColPivot(n,A,degree,2,2);
	else if (args[0].equals("MCColPivotMin3"))    mc = new MCColPivot(n,A,degree,3,2);
	else if (args[0].equals("MCColPivotMax0"))    mc = new MCColPivot(n,A,degree,-0,3);
	else if (args[0].equals("MCColPivotMax-1"))   mc = new MCColPivot(n,A,degree,-1,3);
	else if (args[0].equals("MCColPivotMax-2"))   mc = new MCColPivot(n,A,degree,-2,3);	
	else if (args[0].equals("MCColPivotMax-3"))   mc = new MCColPivot(n,A,degree,-3,3);
	else if (args[0].equals("MCColPivotMax1"))    mc = new MCColPivot(n,A,degree,1,3);
	else if (args[0].equals("MCColPivotMax2"))    mc = new MCColPivot(n,A,degree,2,3);
	else if (args[0].equals("MCColPivotMax3"))    mc = new MCColPivot(n,A,degree,3,3);
	else if (args[0].equals("MCColPivotL0"))    mc = new MCColPivot(n,A,degree,-0,1);
	else if (args[0].equals("MCColPivotL-1"))   mc = new MCColPivot(n,A,degree,-1,1);
	else if (args[0].equals("MCColPivotL-2"))   mc = new MCColPivot(n,A,degree,-2,1);	
	else if (args[0].equals("MCColPivotL-3"))   mc = new MCColPivot(n,A,degree,-3,1);
	else if (args[0].equals("MCColPivotL1"))    mc = new MCColPivot(n,A,degree,1,1);
	else if (args[0].equals("MCColPivotL2"))    mc = new MCColPivot(n,A,degree,2,1);
	else if (args[0].equals("MCColPivotL3"))    mc = new MCColPivot(n,A,degree,3,1);
	else if (args[0].equals("MCQTomita"))       mc = new MCQTomita(n,A,degree,1);
	else if (args[0].equals("MCSaTomita"))      mc = new MCSaTomita(n,A,degree,1);
	else if (args[0].equals("MCMin"))             mc = new MCMin(n,A,degree,1);
	else if (args[0].equals("MCMax"))             mc = new MCMax(n,A,degree,1);
	else return;

	System.gc();
	if (args.length > 2) mc.timeLimit = 1000 * (long)Integer.parseInt(args[2]);
	long cpuTime = System.currentTimeMillis();
	mc.search();
	cpuTime = System.currentTimeMillis() - cpuTime;
	System.out.println(mc.maxSize +" "+ mc.nodes +" "+ cpuTime);
	for (int i=0;i<mc.n;i++) if (mc.solution[i] == 1) System.out.print(i+1 +" ");
	System.out.println();
	System.out.format("%.2f %d",((double)mc.nodesToBest)/((double)mc.nodes),mc.nodesToBest);
	System.out.println();
    }
}
