import java.util.*;
import java.io.*;

public class MaxCliqueEppstein {

    static int[] degree;   // degree of vertices
    static int[][] A;      // 0/1 adjacency matrix
    static int n;

    static void readDIMACS(String fname) throws IOException {
	String s = "";
	Scanner sc = new Scanner(new File(fname));
	while (sc.hasNext() && !s.equals("p")) s = sc.next();
	sc.next();
	n      = sc.nextInt();   
	int m  = sc.nextInt();   
	degree = new int[n];
	A      = new int[n][n];
	while (sc.hasNext()){
	    s     = sc.next(); // skip "edge"
	    int i = sc.nextInt() - 1;
	    int j = sc.nextInt() - 1; 
	    degree[i]++; degree[j]++;
	    A[i][j] = A[j][i] = 1;
	}
	sc.close();
    }

    static void readEppstein(String fname) throws IOException {
	MyIo fin     = new MyIo(fname,", ");
	n            = fin.getNextInt();   
	int m        = fin.getNextInt();   
	degree       = new int[n];
	A            = new int[n][n];
	int maxDeg   = 0;
	int minDeg   = n;
	int isolated = 0; // number of isolated vertices
	for (int k=0;k<m;k++){
	    int i = fin.getNextInt();
	    int j = fin.getNextInt(); 
	    degree[i]++;
	    maxDeg = Math.max(maxDeg,degree[i]);
	    A[i][j] = A[j][i] = 1;
	}
	fin.close();
	for (int i=0;i<n;i++){minDeg = Math.min(minDeg,degree[i]); if (degree[i] == 0) isolated++;}
	//System.out.println("& "+ fname +" & "+ n +" & "+ m/2 +" & "+ minDeg +" & "+ maxDeg +" & "+ isolated);
    }

    public static void main(String[] args)  throws IOException {

	readEppstein(args[1]);
	//System.out.println("reddit");
	MC mc = null;
	if (args[0].equals("MCMin"))       mc = new MCMin(n,A,degree,0);
	else if (args[0].equals("MCMax"))  mc = new MCMax(n,A,degree,0);
	else return;

	System.gc();
	if (args.length > 2) mc.timeLimit = 1000 * (long)Integer.parseInt(args[2]);
	long cpuTime = System.currentTimeMillis();
	mc.search();
	cpuTime = System.currentTimeMillis() - cpuTime;
	System.out.println(mc.maxSize +" "+ mc.nodes +" "+ mc.adjacency +" "+ mc.colourings +" "+ mc.comparisons +" "+ mc.fails);
	System.out.println(mc.searchTime +" "+ mc.sortTime +" "+ mc.cpuTime +" "+ cpuTime);
	//for (int i=0;i<mc.n;i++) if (mc.solution[i] == 1) System.out.print(i+1 +" ");
	//System.out.println();
	System.out.format("%.4f %d  ",((double)mc.nodesToBest)/((double)mc.nodes),mc.nodesToBest);
	System.out.format("%.4f %d  ",((double)mc.adjacencyToBest)/((double)mc.adjacency),mc.adjacencyToBest);
	System.out.println();
    }
}
